program MovingPlasma;
{****************************************************************************
** Demonstration of moving plasma                                          **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************}

uses Crt;

type BufferType = Array [0..63999] of Byte;

var
  {The screen itself}
  Screen  : BufferType absolute $A000:0000;
  {An offscreen buffer}
  Buffer  : ^BufferType;

{Switches to VGA 320x200 256 colour}
procedure VGAScreen; assembler;
asm
  mov ax, $13
  int $10
end;

{Switches to text 80x25}
procedure TextScreen; assembler;
asm
  mov ax, $3
  int $10
end;

{This sets a DAC register to a specific Red Green Blue-value}
procedure SetDAC (DAC, R, G, B : Byte);
begin
  Port [$3C8] := DAC;
  Port [$3C9] := R;
  Port [$3C9] := G;
  Port [$3C9] := B;
end;

{Sets up the (real ugly) palette}
procedure SetPalette;
var
  Counter : Byte;

begin
  for Counter := 0 to 63 do begin
    {Purple}
    SetDAC (Counter      , Counter, 0, Counter);
    SetDAC (Counter +  64, 63 - Counter, 0, 63 - Counter);
    {Yellow}
    SetDAC (Counter + 128, Counter, Counter, 0);
    SetDAC (Counter + 192, 63 - Counter, 63 - Counter, 0);
  end;
end;

{Draws the actual plasma}
procedure DrawPlasma (Phase : Integer);
var
  {Sine waves}
  Hor : Array [0..319] of Integer;
  Ver : Array [0..199] of Integer;
  Pos : Integer;
  {For angles}
  p, t, Radians : Real;
  {Locations on screen}
  x, y : Integer;
  {Offset into video memory}
  o : Word;

begin
  Radians := Phase / 5.0;

  {
    Experiment with the waves, varying the formula the calculates "t"
    to get different kinds of waves.
  }

  {Calculate sine waves}
  for Pos := 0 to 319 do begin
    p := Pos;
    {Put any sine wave you like here}
    t := 35.0 * sin (cos (Radians / 2) * p / 70);
    t := t - 50.0 * cos (Radians + (200 - p) / 40);
    Hor [Pos] := trunc (t) + 100;
  end;

  Radians := Phase / 10.0;
  for Pos := 0 to 199 do begin
    p := Pos;
    {And another one here}
    t := 40.0 * cos (sin (Radians) * p / 25.0);
    t := t + 70.0 * sin (Radians + (200.0 - p) / 30.0);
    Ver [Pos] := trunc (t) + 160;
  end;

  {Draw the entire plasma}
  o := 0;
  for y := 0 to 199 do for x := 0 to 319 do begin
    Buffer^ [o] := (x - Hor [x]) + (y - Ver [y]) + Phase;
    inc (o);
  end;
end;

var
  Frame : Integer;

begin
  {Switch to VGA}
  VGAScreen;

  {I'm drawing to the screen instead of a buffer}
  Buffer := @Screen;

  {Sets an ugly palette}
  SetPalette;

  {Draw the actual plasma}
  repeat
    DrawPlasma (Frame);
    inc (Frame);
  until KeyPressed;

  {And finally, return to text}
  TextScreen;
end.